/***********************************************************************
 *
 * NetSaint CGI Utilities
 * 
 * Written By: Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 07-28-99
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>

#include "cgiutils.h"

char main_config_file[MAX_FILENAME_LENGTH];
char hosts_config_file[MAX_FILENAME_LENGTH];

char log_file[MAX_FILENAME_LENGTH];
char status_file[MAX_FILENAME_LENGTH];

char url_html_path[MAX_FILENAME_LENGTH];
char url_docs_path[MAX_FILENAME_LENGTH];
char url_images_path[MAX_FILENAME_LENGTH];
char url_cgibin_path[MAX_FILENAME_LENGTH];


hostgroup *hostgroup_list=NULL;
hoststatus *hoststatus_list=NULL;
servicestatus *servicestatus_list=NULL;

extern char *tzname[2];

void reset_cgi_vars(void){

	strcpy(main_config_file,"");
	strcpy(hosts_config_file,"");

	strcpy(url_html_path,"");
	strcpy(url_docs_path,"");
	strcpy(url_images_path,"");
	strcpy(url_cgibin_path,"");

	strcpy(log_file,"");
	strcpy(status_file,"");

	hostgroup_list=NULL;
	hoststatus_list=NULL;
	servicestatus_list=NULL;

	return;
        }


void free_memory(void){

	free_hostgroup_memory();
	free_host_status_memory();
	free_service_status_memory();

	return;
        }


void free_hostgroup_memory(void){
	hostgroup *this_hostgroup;
	hostgroup *next_hostgroup;

	/* free memory for the hostgroup list */
	this_hostgroup=hostgroup_list;
	while(this_hostgroup!=NULL){
		next_hostgroup=this_hostgroup->next;
		free(this_hostgroup);
		this_hostgroup=next_hostgroup;
		}

	return;
        }


void free_host_status_memory(void){
	hoststatus *this_hoststatus;
	hoststatus *next_hoststatus;

	/* free memory for the host status list */
	this_hoststatus=hoststatus_list;
	while(this_hoststatus!=NULL){
		next_hoststatus=this_hoststatus->next;
		free(this_hoststatus);
		this_hoststatus=next_hoststatus;
	        }

	return;
        }


void free_service_status_memory(void){
	servicestatus *this_svcstatus;
	servicestatus *next_svcstatus;

	/* free memory for the service status list */
	this_svcstatus=servicestatus_list;
	while(this_svcstatus!=NULL){
		next_svcstatus=this_svcstatus->next;
		free(this_svcstatus);
		this_svcstatus=next_svcstatus;
	        }

	return;
        }


int read_cgi_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;

	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

		if(strstr(input_buffer,"main_config_file=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(main_config_file,temp_buffer);
			strip(main_config_file);
		        }
	        }

	fclose(fp);

	if(!strcmp(main_config_file,""))
		return ERROR;
	else
		return OK;
        }



int read_main_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;

	
	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

		if(strstr(input_buffer,"url_html_path=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strcpy(url_html_path,temp_buffer);
			strip(url_html_path);
			if(url_html_path[strlen(url_html_path)-1]!='/')
				strcat(url_html_path,"/");
			sprintf(url_docs_path,"%sdocs/",url_html_path);
			sprintf(url_images_path,"%simages/",url_html_path);
		        }

		else if(strstr(input_buffer,"url_cgibin_path=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(url_cgibin_path,temp_buffer);
			strip(url_cgibin_path);
			if(url_cgibin_path[strlen(url_cgibin_path)-1]!='/')
				strcat(url_cgibin_path,"/");
		        }


		else if(strstr(input_buffer,"log_file=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(log_file,temp_buffer);
			strip(log_file);
		        }

		else if(strstr(input_buffer,"status_file=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(status_file,temp_buffer);
			strip(status_file);
		        }
	
		else if(strstr(input_buffer,"cfg_file=") && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(hosts_config_file,temp_buffer);
			strip(hosts_config_file);
		        }
                }

	fclose(fp);
	return OK;
        }


int read_hosts_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	hostgroup *new_hostgroup;
	hostgroup *last_hostgroup;
	hostgroup *temp_hostgroup;
	FILE *fp;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

		if(strstr(input_buffer,"hostgroup[")){

			/* add the hostgroup entry to the list in memory */
			new_hostgroup=(hostgroup *)malloc(sizeof(hostgroup));
			if(new_hostgroup!=NULL){
				
				temp_buffer=strtok(input_buffer,"[");
				temp_buffer=strtok(NULL,"]");
				strcpy(new_hostgroup->group_name,temp_buffer);
				temp_buffer=strtok(NULL,";");
				strcpy(new_hostgroup->group_alias,temp_buffer+1);
				temp_buffer=strtok(NULL,";");
				temp_buffer=strtok(NULL,"\x0");
				strcpy(new_hostgroup->members,temp_buffer);
				strip(new_hostgroup->members);

				/* add new hostgroup to hostgroup list, sorted by hostgroup name */
				last_hostgroup=hostgroup_list;
				for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
					if(strcmp(new_hostgroup->group_name,temp_hostgroup->group_name)<0){
						new_hostgroup->next=temp_hostgroup;
						if(temp_hostgroup==hostgroup_list)
							hostgroup_list=new_hostgroup;
						else
							last_hostgroup->next=new_hostgroup;
						break;
					        }
					else
						last_hostgroup=temp_hostgroup;
				        }
				if(hostgroup_list==NULL){
					new_hostgroup->next=NULL;
					hostgroup_list=new_hostgroup;
				        }
				else if(temp_hostgroup==NULL){
					new_hostgroup->next=NULL;
					last_hostgroup->next=new_hostgroup;
				        }

			        }
		        }

	        }

	fclose(fp);

	return OK;
        }


int read_status_log(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	FILE *fp;
	time_t last_check;
	time_t next_check;
	char host_name[MAX_HOSTNAME_LENGTH];
	char service_description[MAX_SERVICEDESC_LENGTH];
	char service_information[MAX_SERVICEINFO_LENGTH];
	char host_information[MAX_HOSTINFO_LENGTH];
	char status[MAX_STATUS_LENGTH];
	int max_attempts;
	int current_attempt;
	char *temp_buffer;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

		if(strstr(input_buffer,"] HOST;")){
		
			temp_buffer=strtok(input_buffer,"]");
			last_check=strtoul(temp_buffer+1,NULL,10);

			temp_buffer=strtok(NULL,";");
			temp_buffer=strtok(NULL,";");
			strcpy(host_name,temp_buffer);

			temp_buffer=strtok(NULL,";");
			strcpy(status,temp_buffer);

			temp_buffer=strtok(NULL,";");
			strcpy(host_information,temp_buffer);

			add_host_status(host_name,status,host_information,last_check);
		        }

		else if(strstr(input_buffer,"] SERVICE;")){

			temp_buffer=strtok(input_buffer,"]");
			last_check=strtoul(temp_buffer+1,NULL,10);

			temp_buffer=strtok(NULL,";");
			temp_buffer=strtok(NULL,";");
			strcpy(host_name,temp_buffer);
			
			temp_buffer=strtok(NULL,";");
			strcpy(service_description,temp_buffer);

			temp_buffer=strtok(NULL,";");
			strcpy(status,temp_buffer);
			
			temp_buffer=strtok(NULL,"/");
			current_attempt=atoi(temp_buffer);

			temp_buffer=strtok(NULL,";");
			max_attempts=atoi(temp_buffer);
			
			temp_buffer=strtok(NULL,";");
			next_check=strtoul(temp_buffer,NULL,10);

			temp_buffer=strtok(NULL,";");
			strcpy(service_information,temp_buffer);

			add_service_status(host_name,service_description,status,current_attempt,max_attempts,service_information,last_check,next_check);
		        }
	        }

	fclose(fp);

	return OK;
        }


void add_host_status(char *host_name, char *status, char *host_information, time_t last_check){
	hoststatus *new_status;
	hoststatus *last_status;
	hoststatus *temp_status;

	/* allocate memory for new host status */
	new_status=(hoststatus *)malloc(sizeof(hoststatus));
	if(new_status==NULL)
		return;

	strcpy(new_status->host_name,host_name);
	if(!strcmp(status,"DOWN"))
		new_status->status=HOST_DOWN;
	else if(!strcmp(status,"UNREACHABLE"))
		new_status->status=HOST_UNREACHABLE;
	else if(!strcmp(status,"PENDING"))
		new_status->status=HOST_PENDING;
	else
		new_status->status=HOST_UP;
	strcpy(new_status->information,host_information);
	new_status->last_check=last_check;

	/* add new host status to list, sorted by host name */

	last_status=hoststatus_list;
	for(temp_status=hoststatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(strcmp(new_status->host_name,temp_status->host_name)<0){
			new_status->next=temp_status;
			if(temp_status==hoststatus_list)
				hoststatus_list=new_status;
			else
				last_status->next=new_status;
			break;
		        }
		else
			last_status=temp_status;
	        }
	if(hoststatus_list==NULL){
		new_status->next=NULL;
		hoststatus_list=new_status;
	        }
	else if(temp_status==NULL){
		new_status->next=NULL;
		last_status->next=new_status;
	        }
	
	return;
        }


void add_service_status(char *host_name, char *service_description, char *status, int current_attempt, int max_attempts, char *service_information, time_t last_check, time_t next_check){
	servicestatus *new_status;
	servicestatus *last_status;
	servicestatus *temp_status;


	/* allocate memory for a new service status entry */
	new_status=(servicestatus *)malloc(sizeof(servicestatus));
	if(new_status==NULL)
		return;

	strcpy(new_status->host_name,host_name);
	strcpy(new_status->description,service_description);
	strcpy(new_status->information,service_information);
	new_status->current_attempt=current_attempt;
	new_status->max_attempts=max_attempts;
	new_status->last_check=last_check;
	new_status->next_check=next_check;
	if(!strcmp(status,"WARNING"))
		new_status->status=SERVICE_WARNING;
	else if(!strcmp(status,"UNKNOWN"))
		new_status->status=SERVICE_UNKNOWN;
	else if(!strcmp(status,"CRITICAL"))
		new_status->status=SERVICE_CRITICAL;
	else if(!strcmp(status,"RECOVERY"))
		new_status->status=SERVICE_RECOVERY;
	else if(!strcmp(status,"HOST DOWN"))
		new_status->status=SERVICE_HOST_DOWN;
	else if(!strcmp(status,"UNREACHABLE"))
		new_status->status=SERVICE_UNREACHABLE;
	else if(!strcmp(status,"PENDING"))
		new_status->status=SERVICE_PENDING;
	else
		new_status->status=SERVICE_OK;

	/* add new service status to front of list */
	last_status=servicestatus_list;
	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(strcmp(new_status->host_name,temp_status->host_name)<0){
			new_status->next=temp_status;
			if(temp_status==servicestatus_list)
				servicestatus_list=new_status;
			else
				last_status->next=new_status;
			break;
		        }
		else
			last_status=temp_status;
	        }
	if(servicestatus_list==NULL){
		new_status->next=NULL;
		servicestatus_list=new_status;
	        }
	else if(temp_status==NULL){
		new_status->next=NULL;
		last_status->next=new_status;
	        }

	return;
        }


int get_host_count(char *host_name, int type){
	hoststatus *temp_status;
	int count=0;

	for(temp_status=hoststatus_list;temp_status!=NULL;temp_status=temp_status->next)
		if((temp_status->status & type) && (host_name==NULL || !strcmp(host_name,temp_status->host_name)))
			count++;

	return count;
        }

int get_service_count(char *host_name, int type){
	servicestatus *temp_status;
	int count=0;

	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next)
		if((temp_status->status & type) && (host_name==NULL || !strcmp(host_name,temp_status->host_name)))
			count++;

	return count;
        }


int is_member_of_hostgroup(hostgroup *hg, char *host_name){
	char members[MAX_INPUT_BUFFER];
	char *temp_buffer;

	strcpy(members,hg->members);

	for(temp_buffer=strtok(members,",");temp_buffer!=NULL;temp_buffer=strtok(NULL,","))
		if(!strcmp(temp_buffer,host_name))
			return TRUE;
	

	return FALSE;
        }


/* recursively strips newline characters, spaces, and carriage returns from the end of a string */
void strip(char *buffer){
	int x;

	x=strlen(buffer);

	if(!x)
		return;
	else
		x--;

	if(buffer[x]==' ' || buffer[x]=='\n' || buffer[x]=='\r' || buffer[x]=='\t' || buffer[x]==13)
		buffer[x]='\x0';

	else
		return;

	strip(buffer);

	return;
        }


/* reads a line of text from a file and strips it clean */
char * read_raw_line(char *buffer, int maxlength, FILE *fp){
	char *result;

	/* clear the buffer */
	strcpy(buffer,"");

	/* read in one line of text */
	result=fgets(buffer,maxlength-1,fp);

	/* if we were able to read something, strip extra characters from the end */
	if(result)
		strip(buffer);

	/* return the number of characters read */
	return result;
        }


/* read a line of text from a file, skipping comments and empty lines */
char * read_line(char *buffer, int maxlength, FILE *fp){
	char *result;
	int keep_line=TRUE;

	while((result=read_raw_line(buffer,maxlength,fp))){

		if(buffer[0]=='#')
			keep_line=FALSE;
		else if(buffer[0]=='\r')
			keep_line=FALSE;
		else if(buffer[0]=='\n')
			keep_line=FALSE;
		else if(buffer[0]=='\x0')
			keep_line=FALSE;
		
		if(keep_line==TRUE)
			break;

		else
			keep_line=TRUE;
			
	        }

	return result;
        }


hostgroup * find_hostgroup(char *hostgroup_name){
	hostgroup *temp_hostgroup;

	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
		if(!strcmp(hostgroup_name,temp_hostgroup->group_name))
			return temp_hostgroup;
	        }
	
	return NULL;
        }


void get_time_string(unsigned long *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};

	if(raw_time==NULL)
		time(&t);
	else 
		t=(time_t)(*raw_time);

	tm_ptr=localtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

	snprintf(buffer,buffer_length,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,tzname[1],year);

	return;
        }
