/***********************************************************************
 *
 * HISTORY.C - NetSaint History CGI
 *
 * Written by Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 07-25-1999
 *
 * This CGI program will display the history for the specified host.
 * If no host is specified, the history for all hosts will be displayed.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "getcgi.h"
#include "cgiutils.h"
#include "../common/common.h"

#define DISPLAY_HOSTS			0
#define DISPLAY_HOSTGROUPS		1

void get_history(void);

void document_header(void);
void document_footer(void);
int process_cgivars(void);

extern char main_config_file[MAX_FILENAME_LENGTH];
extern char url_images_path[MAX_FILENAME_LENGTH];
extern char log_file[MAX_FILENAME_LENGTH];

int show_all_hosts=FALSE;
int show_all_hostgroups=FALSE;
char host_name[MAX_HOSTNAME_LENGTH]="";
char hostgroup_name[MAX_HOSTGROUPNAME_LENGTH]="";
int display_type=DISPLAY_HOSTS;



int main(void){
	int error=FALSE;
	int result=OK;
	time_t t;
	char date_time[48];

	/* get the variables passed to us */
	error=process_cgivars();

	/* if there was an error with the variables, print out an error message */
	if(error==TRUE){
		document_header();
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Invalid argument supplied!");
		printf("</strong></font></div></p>\n");
		document_footer();
	        }

	else{

		document_header();

		printf("<p><div align=\"center\">\n");

		if(display_type==DISPLAY_HOSTS){
			if(show_all_hosts==TRUE)
				printf("<h2>History For All Hosts</h2>\n");
			else
				printf("<h2>History For Host \"%s\"</h2>\n",host_name);
		        }
		else{
			if(show_all_hostgroups==TRUE)
				printf("<h2>History For All Host Groups</h2>\n");
			else
				printf("<h2>History For Host Group \"%s\"</h2>\n",hostgroup_name);
		        }

		time(&t);
		printf("<font size=-1>\n");
		get_time_string(&t,date_time,sizeof(date_time));
		printf("Last Updated: %s",date_time);
		printf("</font>\n");
		printf("</div></p>\n");
	
		reset_cgi_vars();

		result=read_cgi_config_file(CGI_CONFIG_FILE);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open CGI configuration file for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		result=read_main_config_file(main_config_file);
		if(result==ERROR){
			printf("<p><div align=\"center\"><font color=\"red\"><strong>\n");
			printf("Error: Could not open main configuration file for reading!\n");
			printf("</strong></font></div></p>\n");
			document_footer();
			return ERROR;
	                }

		get_history();

		document_footer();
	        }

	return OK;
        }


void get_history(void){
	FILE *fp;
	char image[MAX_INPUT_BUFFER];
	char image_alt[MAX_INPUT_BUFFER];
	char input_buffer[MAX_INPUT_BUFFER];
	char match1[MAX_INPUT_BUFFER];
	char match2[MAX_INPUT_BUFFER];
	int found_line=FALSE;
	int system_message=FALSE;
	int display_line=FALSE;
	time_t t;
	char date_time[48];
	char *temp_buffer;

	fp=fopen(log_file,"r");
	if(fp==NULL){
		printf("<HR>\n");
		printf("<p><div align=\"center\"><font color=\"red\"><strong>");
		printf("Error: Cannot open log file for reading!");
		printf("</strong></font></div></p>\n");
		printf("<HR>\n");
		return;
	        }

	printf("<HR>\n");

	printf("<P>\n");
	printf("<font size=-1>\n");

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		strcpy(image,"");
		strcpy(image_alt,"");
		system_message=FALSE;

		if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";CRITICAL;")){
			strcpy(image,CRITICAL_ICON);
			strcpy(image_alt,CRITICAL_ICON_ALT);
                        }
		else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";WARNING;")){
			strcpy(image,WARNING_ICON);
			strcpy(image_alt,WARNING_ICON_ALT);
                        }
		else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";UNKNOWN;")){
			strcpy(image,UNKNOWN_ICON);
			strcpy(image_alt,UNKNOWN_ICON_ALT);
                        }
		else if(strstr(input_buffer,"SERVICE ALERT:") && strstr(input_buffer,";RECOVERY;")){
			strcpy(image,RECOVERY_ICON);
			strcpy(image_alt,RECOVERY_ICON_ALT);
                        }
		else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";DOWN")){
			strcpy(image,HOST_CRITICAL_ICON);
			strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
		        }
		else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";UNREACHABLE")){
			strcpy(image,HOST_CRITICAL_ICON);
			strcpy(image_alt,HOST_CRITICAL_ICON_ALT);
		        }
		else if(strstr(input_buffer,"HOST ALERT:") && strstr(input_buffer,";RECOVERY")){
			strcpy(image,HOST_RECOVERY_ICON);
			strcpy(image_alt,HOST_RECOVERY_ICON_ALT);
		        }
		else if(strstr(input_buffer," starting...")){
			strcpy(image,START_ICON);
			strcpy(image_alt,START_ICON_ALT);
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," shutting down...")){
			strcpy(image,STOP_ICON);
			strcpy(image_alt,STOP_ICON_ALT);
			system_message=TRUE;
		        }
		else if(strstr(input_buffer," restarting...")){
			strcpy(image,RESTART_ICON);
			strcpy(image_alt,RESTART_ICON_ALT);
			system_message=TRUE;
		        }

		/* get the timestamp */
		temp_buffer=strtok(input_buffer,"]");
		t=strtoul(temp_buffer+1,NULL,10);
		get_time_string(&t,date_time,sizeof(date_time));
		strip(date_time);

		temp_buffer=strtok(NULL,"\n");

		if(strcmp(image,"")){

			display_line=FALSE;

			if(system_message==TRUE)
				display_line=TRUE;
			else if(display_type==DISPLAY_HOSTS){
				sprintf(match1," HOST ALERT: %s;",host_name);
				sprintf(match2," SERVICE ALERT: %s;",host_name);
				if(show_all_hosts==TRUE)
					display_line=TRUE;
				else if(strstr(temp_buffer,match1))
					display_line=TRUE;
				else if(strstr(temp_buffer,match2))
					display_line=TRUE;
			        }
			else if(display_type==DISPLAY_HOSTGROUPS){
				if(show_all_hostgroups==TRUE)
					display_line=TRUE;
			        }
			
			if(display_line==TRUE){
				printf("<img align=\"left\" src=\"%s%s\" alt=\"%s\">[%s] %s<br clear=\"all\">\n",url_images_path,image,image_alt,date_time,temp_buffer);
				found_line=TRUE;
			        }
		        }

                }

	if(found_line==FALSE){
		printf("<div align=\"center\"><font color=\"red\"><strong>");
		printf("No history is available yet");
		printf("</strong></font></div>\n");
	        }

	printf("</FONT>\n");
	printf("</P>\n");
	
	printf("<HR>\n");

	fclose(fp);

	return;
        }


void document_header(void){

	printf("Content-type: text/html\r\n\r\n");

	printf("<html>\n");
	printf("<head>\n");
	printf("<META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">\n");
	printf("<title>\n");
	printf("NetSaint History\n");
	printf("</title>\n");
	printf("</head>\n");
	printf("<body>\n");

	return;
        }


void document_footer(void){

	printf("</body>\n");
	printf("</html>\n");

	return;
        }


int process_cgivars(void){
	char **variables;
	int error=FALSE;
	int x;

	variables=getcgivars();

	for(x=0;variables[x]!=NULL;x++){

		/* do some basic length checking on the variable identifier to prevent buffer overflows */
		if(strlen(variables[x])>=MAX_INPUT_BUFFER-1){
			x++;
			continue;
		        }

		/* we found the host argument */
		else if(!strcmp(variables[x],"host")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTNAME_LENGTH-1){
				strcpy(host_name,"all");
			        }
			else
				strcpy(host_name,variables[x]);

			display_type=DISPLAY_HOSTS;

			if(!strcmp(host_name,"all"))
				show_all_hosts=TRUE;
			else
				show_all_hosts=FALSE;
		        }

		/* we found the hostgroup argument */
		else if(!strcmp(variables[x],"hostgroup")){
			x++;
			if(variables[x]==NULL){
				error=TRUE;
				break;
			        }

			/* protect against buffer overflows */
			if(strlen(variables[x])>=MAX_HOSTGROUPNAME_LENGTH-1){
				strcpy(hostgroup_name,"all");
			        }
			else
				strcpy(hostgroup_name,variables[x]);

			display_type=DISPLAY_HOSTGROUPS;

			if(!strcmp(hostgroup_name,"all"))
				show_all_hostgroups=TRUE;
			else
				show_all_hostgroups=FALSE;
		        }
	
	        }

	return error;
        }

